# Copyright (c) 2011 Cloudera, Inc. All rights reserved.

import logging

from cmf.monitor.abstract_monitor import AbstractMonitor
from cmf.monitor.monitor_properties import ConfParser

from cmf.url_util import urlopen_with_timeout
from cmf.monitor import constants

_HOST = 'host'
_SERVICE_VERSION = 'service_version'

LOG = logging.getLogger(__name__)

class AbstractHdfsMonitor(AbstractMonitor):
  """
  A common AbstractMonitor subclass for HDFS role monitors.
  """
  def __init__(self, daemon, registry, section, metric_port_name):
    AbstractMonitor.__init__(self, daemon, registry)
    self._section = section
    self._metric_port_name = metric_port_name
    self._metrics_url = None

  # protected
  def _get_metrics_url(self):
    parser = self.conf_parser
    return self._compute_metrics_url(
      parser.getint_with_default(self._section, _SERVICE_VERSION, 
                                 constants.SERVICE_VERSION_CDH3),
      parser.get_with_default(self._section, _HOST, None),
      parser.getint_with_default(self._section, self._metric_port_name, None))

  # protected
  def _compute_metrics_url(self, version, host, port):
    if host is None or port is None:
      return None
    if version == constants.SERVICE_VERSION_CDH3 or \
       version == constants.SERVICE_VERSION_CDH3U4X:
      return "http://%s:%s/metrics?format=json" % (host, port)
    elif version == constants.SERVICE_VERSION_CDH4:
      return "http://%s:%s/jmx" % (host, port)
    else:
      LOG.error("Unsupported service version: %d", version)
      return None

  def is_ready_to_report(self):
    if self._pid is None:
      return False
    if not self.is_metrics_url_responding(self._get_metrics_url(), 0.5):
      return False
    return True

  @property
  def metrics_url(self):
    return self._metrics_url

